package com.supervision.knowsub.controller.system;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.exception.BusinessException;
import com.supervision.knowsub.service.UserManageService;
import com.supervision.knowsub.vo.user.*;
import com.supervision.knowsub.dto.user.UserInfoDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

@Tag(name = "用户管理")
@RestController
@RequestMapping("user")
@RequiredArgsConstructor
public class UserController {

    private final UserManageService userManageService;

    @Operation(summary = "登录")
    @PostMapping("login")
    public LoginResVO login(@RequestBody LoginReqVO reqVO) {
        if (!StrUtil.isAllNotBlank(reqVO.getUserAccount(), reqVO.getPassword())) {
            throw new BusinessException("用户名不能为空");
        }
        return userManageService.login(reqVO);
    }

    @Operation(summary = "查看账号信息")
    @GetMapping("getUserAccountInfo")
    public UserInfoDTO getUserAccountInfo() {

        return userManageService.getUserAccountInfo();
    }


    @Operation(summary = "新增用户信息")
    @PostMapping("/saveUserInfo")
    public String saveUserInfo(@RequestBody UserInfoReqVo userInfoReqVo) {

        return userManageService.saveUserInfo(userInfoReqVo);
    }

    @Operation(summary = "修改用户信息")
    @PostMapping("/update")
    public void updateUser(@RequestBody UserInfoReqVo userInfoReqVo) {
        userManageService.updateUserInfo(userInfoReqVo);
    }

    @Operation(summary = "修改用户状态")
    @PostMapping("/updateUserStatus")
    public void updateUserStatus(@RequestBody UserStatusReqVo userStatusReqVo) {
        userManageService.updateUserStatus(userStatusReqVo);
    }

    @Operation(summary = "删除用户信息")
    @DeleteMapping("/delete")
    public Boolean deleteUser(@Parameter(name = "id") @RequestParam(name="id") String id) {
        return userManageService.deleteUser(id);
    }
    @Operation(summary = "查看用户信息列表")
    @GetMapping("/list")
    public IPage<UserInfoDTO> list(@Parameter(name = "userName",description = "用户名") @RequestParam(required = false) String userName,
                                  @Parameter(name = "deptId",description = "部门id") @RequestParam(required = false) String deptId,
                                  @Parameter(name = "roleId",description = "角色id") @RequestParam(required = false) String roleId,
                                  @Parameter(name = "roleName",description = "角色名") @RequestParam(required = false) String roleName,
                                  @Parameter(name = "pageNum",description = "页码") @RequestParam(defaultValue = "1") Integer pageNum,
                                  @Parameter(name = "pageSize",description = "每页大小") @RequestParam(defaultValue = "10") Integer pageSize) {
        return userManageService.list(userName, deptId, roleId, roleName, pageNum, pageSize);
    }



    @Operation(summary = "重置用户密码")
    @PutMapping("/resetPW")
    public Boolean resetPW(@RequestBody UserPwReqVo userPwReqVo) {
        return userManageService.resetPW(userPwReqVo);
    }


}
