package com.supervision.knowsub.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.knowsub.model.SystemRole;
import com.supervision.knowsub.service.RoleManageService;
import com.supervision.knowsub.service.SystemRoleService;
import com.supervision.knowsub.vo.role.RoleReqVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class RoleManageServiceImpl implements RoleManageService {

    private final SystemRoleService systemRoleService;
    @Override
    public String saveRole(RoleReqVo roleReqVo) {

        Assert.notEmpty(roleReqVo.getRoleName(), "角色名称不能为空");
        Long count = systemRoleService.lambdaQuery().eq(SystemRole::getRoleName, roleReqVo.getRoleName()).count();
        Assert.isTrue(count == 0, "角色名称已存在");

        // 保存角色
        SystemRole systemRole = new SystemRole();
        systemRole.setRoleName(roleReqVo.getRoleName());
        systemRole.setRemark(roleReqVo.getMark());
        systemRole.setRoleType(null == roleReqVo.getManage() ? 0 : roleReqVo.getManage());
        systemRoleService.save(systemRole);

        return systemRole.getId();
    }

    @Override
    public void updateRole(RoleReqVo roleReqVo) {
        Assert.notEmpty(roleReqVo.getId(), "角色id不能为空");
        Assert.notEmpty(roleReqVo.getRoleName(), "角色名称不能为空");

        SystemRole dbSystemRole = systemRoleService.getById(roleReqVo.getId());
        Assert.notNull(dbSystemRole, "角色不存在");
        if (!StrUtil.equals(dbSystemRole.getRoleName(), roleReqVo.getRoleName())){
            Long count = systemRoleService.lambdaQuery().eq(SystemRole::getRoleName, roleReqVo.getRoleName()).count();
            Assert.isTrue(count == 0, "角色名称已存在");
        }

        dbSystemRole.setRoleName(roleReqVo.getRoleName());
        dbSystemRole.setRemark(roleReqVo.getMark());
        dbSystemRole.setRoleType(null == roleReqVo.getManage() ? 0 : roleReqVo.getManage());

        systemRoleService.updateById(dbSystemRole);
    }

    @Override
    public Boolean deleteRole(String id) {

        Assert.notEmpty(id, "角色id不能为空");

        return systemRoleService.removeById(id);
    }

    @Override
    public IPage<SystemRole> list(String roleName, Integer pageNum, Integer pageSize) {

        return systemRoleService.lambdaQuery().like(StrUtil.isNotEmpty(roleName), SystemRole::getRoleName, roleName)
                .orderBy(true,false ,SystemRole::getUpdateTime).page(new Page<>(pageNum, pageSize));

    }
}
