package som.supervision.knowsub.config;

import org.springframework.ai.vectorstore.RedisVectorStore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.Assert;

@Configuration
@EnableConfigurationProperties({RedisVectorProperties.class, EmbeddingProperties.class})
public class RedisVectorStoreConfig {

    @Bean
    @ConditionalOnProperty(prefix = "embedding", name = "url")
    public VectorEmbeddingClient vectorEmbeddingClient(EmbeddingProperties embeddingProperties) {
        Assert.notNull(embeddingProperties.getUrl(), "配置文件embedding:url未找到");
        return new VectorEmbeddingClient(embeddingProperties.getUrl());
    }

    @Bean
    @ConditionalOnProperty(prefix = "vector.redis", name = "uri")
    public RedisVectorStore redisVectorStore(VectorEmbeddingClient vectorEmbeddingClient, RedisVectorProperties redisVectorProperties) {
        Assert.notNull(redisVectorProperties.getUri(), "配置文件vector.redis.uri未找到");
        RedisVectorStore.RedisVectorStoreConfig config = RedisVectorStore.RedisVectorStoreConfig.builder()
                .withURI(redisVectorProperties.getUri())
                .withPrefix(redisVectorProperties.getPrefix())
                .withIndexName(redisVectorProperties.getIndexName())
                // 定义搜索过滤器使用的元数据字段(!!!!!!!!千万重要,数据类型一定要用字符串,否则会导致查询不到!!!!!!!!)
                .withMetadataFields(
                        RedisVectorStore.MetadataField.tag("fileName"))
                .build();
        return new RedisVectorStore(config, vectorEmbeddingClient);
    }
}
