package som.supervision.knowsub.service.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.document.Document;
import org.springframework.ai.reader.tika.TikaDocumentReader;
import org.springframework.ai.transformer.splitter.TokenTextSplitter;
import org.springframework.ai.vectorstore.ElasticsearchVectorStore;
import org.springframework.ai.vectorstore.RedisVectorStore;
import org.springframework.core.io.InputStreamResource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import som.supervision.knowsub.service.KnowledgeEtlService;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

@Slf4j
@Service
@RequiredArgsConstructor
public class KnowledgeEtlServiceImpl implements KnowledgeEtlService {

    private final ElasticsearchVectorStore elasticsearchVectorStore;

    private final RedisVectorStore redisVectorStore;

    /**
     * 参考文档 <a href="https://zhuanlan.zhihu.com/p/703705663"/>
     *
     * @param inputStream 输入流
     */
    private void loadFile(InputStream inputStream, String fileName) {
        // 首先使用tika进行文件切分操作
        log.info("{} 进行内容切分", fileName);
        TikaDocumentReader tikaDocumentReader = new TikaDocumentReader(new InputStreamResource(inputStream));
        List<Document> documents = tikaDocumentReader.read();
        log.info("{} 切分完成,开始进行chunk分割", fileName);
        // 然后切分为chunk
        TokenTextSplitter tokenTextSplitter = new TokenTextSplitter(500, 250, 10, 1000, true);
        List<Document> apply = tokenTextSplitter.apply(documents);
        for (Document document : apply) {
            document.getMetadata().put("fileName", fileName);
        }
        log.info("{} 切分完成,开始进行保存到向量库中", fileName);
        // 保存到向量数据库中
        elasticsearchVectorStore.accept(apply);
        log.info("{} 保存完成", fileName);

    }


    @Override
    public void knowledgeEtl(MultipartFile[] files) {
        AtomicInteger atomicInteger = new AtomicInteger(1);
        for (MultipartFile file : files) {
            try {
                loadFile(file.getInputStream(), file.getOriginalFilename());
            } catch (Exception e) {
                log.error("{}文件处理失败", file.getOriginalFilename(), e);
            }
            int andIncrement = atomicInteger.getAndIncrement();
            log.info("处理第{}个文件,剩余:{}个", andIncrement, files.length - andIncrement + 1);

        }
        log.info("文件处理结束");
    }

    @Override
    public void redisKnowledgeEtl(MultipartFile[] files) {
        AtomicInteger atomicInteger = new AtomicInteger(1);
        for (MultipartFile file : files) {
            try {
                // 首先使用tika进行文件切分操作
                log.info("{} 进行内容切分", file.getOriginalFilename());
                TikaDocumentReader tikaDocumentReader = new TikaDocumentReader(new InputStreamResource(file.getInputStream()));
                List<Document> documents = tikaDocumentReader.read();
                log.info("{} 切分完成,开始进行chunk分割", file.getOriginalFilename());
                // 然后切分为chunk
                TokenTextSplitter tokenTextSplitter = new TokenTextSplitter(500, 250, 10, 1000, true);
                List<Document> apply = tokenTextSplitter.apply(documents);
                for (Document document : apply) {
                    document.getMetadata().put("fileName", file.getOriginalFilename());
                }
                log.info("{} 切分完成,开始进行保存到向量库中", file.getOriginalFilename());
                // 保存到向量数据库中
                redisVectorStore.accept(apply);
                log.info("{} 保存完成", file.getOriginalFilename());
            } catch (Exception e) {
                log.error("{}文件处理失败", file.getOriginalFilename(), e);
            }
            int andIncrement = atomicInteger.getAndIncrement();
            log.info("处理第{}个文件,剩余:{}个", andIncrement, files.length - andIncrement + 1);

        }
        log.info("文件处理结束");
    }
}
