package com.supervision.knowsub.model;

import com.baomidou.mybatisplus.annotation.*;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.LocalDateTime;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

/**
 * 知识表
 *
 * @TableName ks_knowledge
 */
@TableName(value = "ks_knowledge")
@Data
@Schema(description = "知识库对象")
public class Knowledge implements Serializable {
    /**
     * 主键
     */
    @TableId
    @Schema(description = "主键")
    private String id;

    /**
     * 知识标题
     */
    @Schema(description = "知识标题")
    private String title;

    /**
     * 摘要
     */
    @Schema(description = "摘要")
    private String excerpt;

    /**
     * 知识内容ID
     */
    @Schema(description = "知识内容ID")
    private String contentId;

    /**
     * 知识基本信息ID
     */
    @Schema(description = "知识基本信息ID")
    private String infoId;

    /**
     * 应用子库ID
     */
    @Schema(description = "应用子库ID")
    private String baseId;

    /**
     * 发文部门ID
     */
    @Schema(description = "发文部门ID")
    private String publishDeptId;

    /**
     * 知识来源 1人工添加
     */
    @Schema(description = "知识来源 1人工添加")
    private Integer knowledgeFrom;

    /**
     * 流转状态
     */
    @Schema(description = "流转状态 1草稿 2通过 3驳回 4已撤回 5已失效 6已删除 10待审批(新增) 11待审批(删除) 12待审批(撤回)")
    private Integer status;

    /**
     * 报送部门ID
     */
    @Schema(description = "报送部门ID")
    private String submitDeptId;

    @Schema(description = "报送时间(最后一次审批通过的时间)")
    private LocalDate submitDate;

    @Schema(description = "草稿归属人ID,注意,在发布之后或撤回之后应该及时修改")
    private String draftBelongUserId;

    /**
     * 创建人ID
     */
    @TableField(fill = FieldFill.INSERT)
    private String createUserId;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    /**
     * 更新人
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateUserId;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;

    @TableField(exist = false)
    private static final long serialVersionUID = 1L;
}