package com.supervision.knowsub.config;

import org.elasticsearch.client.RestClient;
import org.springframework.ai.autoconfigure.vectorstore.elasticsearch.ElasticsearchVectorStoreProperties;
import org.springframework.ai.embedding.EmbeddingModel;
import org.springframework.ai.vectorstore.ElasticsearchVectorStore;
import org.springframework.ai.vectorstore.ElasticsearchVectorStoreOptions;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.Assert;

@Configuration
@EnableConfigurationProperties(EmbeddingProperties.class)
public class ElasticsearchVectorStoreConfig {

    @Bean
    @ConditionalOnProperty(prefix = "embedding", name = "url")
    public EmbeddingModel embeddingModel(EmbeddingProperties embeddingProperties) {
        Assert.notNull(embeddingProperties.getUrl(), "配置文件embedding:url未找到");
        return new VectorEmbeddingModel(embeddingProperties.getUrl());
    }

    @Bean
    @ConditionalOnProperty(prefix = "embedding", name = "url")
    public ElasticsearchVectorStore vectorStore(ElasticsearchVectorStoreProperties properties,EmbeddingModel embeddingModel, RestClient restClient) {
        ElasticsearchVectorStoreOptions options = new ElasticsearchVectorStoreOptions();
        options.setIndexName(properties.getIndexName());
        options.setDimensions(1024);
        return new ElasticsearchVectorStore(options, restClient, embeddingModel, true);
    }

}
