package com.supervision.knowsub.service.impl;

import cn.hutool.core.util.StrUtil;
import com.supervision.knowsub.service.RagService;
import com.supervision.knowsub.vo.RagResVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.model.ChatResponse;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.chat.prompt.SystemPromptTemplate;
import org.springframework.ai.document.Document;
import org.springframework.ai.ollama.OllamaChatModel;
import org.springframework.ai.vectorstore.ElasticsearchVectorStore;
import org.springframework.ai.vectorstore.RedisVectorStore;
import org.springframework.ai.vectorstore.SearchRequest;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class RagServiceImpl implements RagService {

    private final ElasticsearchVectorStore elasticsearchVectorStore;

    private final RedisVectorStore redisVectorStore;

//    private final OllamaChatClient chatClient ;

    private final OllamaChatModel ollamaChatModel;


    private static final String springDemoSystemPrompt = """
            您正在协助解答有关卡琳娜所提供服务的问题。
            卡琳娜是一个政府政策解决中心，专注于政策问题解答。
            卡琳娜的使命是根据政策文档，尽可能回答用户关于政策上的疑问。
                 
            利用来自“上下文”部分的信息提供准确的答案。
            您只提供重新表述的查询，且别无其他！不要在查询前后添加任何内容！
            如果不确定，只需说明您不知道。
                        
            政策内容:
            <context>{context}</context>
            """;

    private static final String systemPrompt = """
            使用下面的语料来回答问题。如果你不知道问题的答案，直接回答"我不知道"，禁止随意编造答案。
            保证答案尽可能简洁，你的回答中不可以带有星号。请注意!在每次回答结束之后，你都必须接上"感谢你的提问"作为结束语.
            您只提供重新表述的查询，且别无其他！不要在查询前后添加任何内容！
            以下是一对问题和答案的样例:请问:秦始皇的原名是什么?答:秦始皇原名赢政。感谢你的提问。
            以下是语料:
            <context>{context}</context>
            """;

    public static final String systemPrompt1 = """
            您是政务知识领域的权威专家，能够根据给定的内容，为用户从给定内容中提炼出问题的准确、详尽、合规且具有权威性的回答。
            若给定内容中信息不足或不明确，应向用户说明情况，并告知无法给出确切答案的原因。
                        
            回答内容中,不要出现:"请注意，具体的政策和流程可能会有所变化，因此建议您咨询当地的人力资源和社会保障部门或访问官方网站以获取最新信息。"!
            回答内容中也不要出现"根据您提供的信息"字样!
                        
            以下是参考资料,请围绕参考资料回答:
            <context>{context}</context>
            """;

    public static final String langChainChatPrompt = """
            <指令>根据已知信息，简洁和专业的来回答问题。如果无法从中得到答案，请说 "根据已知信息无法回答该问题"，不允许在答案中添加编造成分，答案请使用中文。 </指令>
            <已知信息>{{ context }}</已知信息>
            <问题>{{ question }}</问题>
            """;

    @Override
    public RagResVO esAsk(String question) {
        log.info("检索相关文档");
        List<Document> similarDocuments = elasticsearchVectorStore.similaritySearch(SearchRequest.query(question).withTopK(10).withSimilarityThreshold(0.5));
        Set<String> fileNameList = new HashSet<>();
        for (Document similarDocument : similarDocuments) {
            fileNameList.add(String.valueOf(similarDocument.getMetadata().get("fileName")));
        }
        log.info("找到:{}条相关文档", similarDocuments.size());
        // 构建系统消息
//        String relevantDocument = similarDocuments.stream().map(Document::getContent).collect(Collectors.joining("\n"));
//        SystemPromptTemplate systemPromptTemplate = new SystemPromptTemplate(systemPrompt1);
//        Message systemMessage = systemPromptTemplate.createMessage(Map.of("context", relevantDocument));
//        // 构建用户消息
//        UserMessage userMessage = new UserMessage(question);
//        Prompt prompt = new Prompt(List.of(systemMessage, userMessage));

        // 构建系统消息
        String relevantDocument = similarDocuments.stream().map(Document::getContent).collect(Collectors.joining("\n"));
        String format = StrUtil.format(langChainChatPrompt, Map.of("context", relevantDocument, "question", question));

        Prompt prompt = new Prompt(new UserMessage(format));

        log.info("开始询问GPT问题");
        ChatResponse call = ollamaChatModel.call(prompt);
        log.info("AI responded.");
        RagResVO ragResVO = new RagResVO();
        ragResVO.setAnswer(call.getResult().getOutput().getContent());
        ragResVO.setFileName(fileNameList);


        return ragResVO;
    }

    @Override
    public RagResVO redisAsk(String question) {
        log.info("检索相关文档");
        List<Document> similarDocuments = redisVectorStore.similaritySearch(SearchRequest.query(question).withTopK(10).withSimilarityThreshold(0.5));
        Set<String> fileNameList = new HashSet<>();
        for (Document similarDocument : similarDocuments) {
            fileNameList.add(String.valueOf(similarDocument.getMetadata().get("fileName")));
        }
        log.info("找到:{}条相关文档", similarDocuments.size());
//        // 构建系统消息
//        String relevantDocument = similarDocuments.stream().map(Document::getContent).collect(Collectors.joining("\n"));
//        SystemPromptTemplate systemPromptTemplate = new SystemPromptTemplate(systemPrompt1);
//        Message systemMessage = systemPromptTemplate.createMessage(Map.of("context", relevantDocument));
//        // 构建用户消息
//        UserMessage userMessage = new UserMessage(question);
//        Prompt prompt = new Prompt(List.of(systemMessage, userMessage));

        // 构建系统消息
        String relevantDocument = similarDocuments.stream().map(Document::getContent).collect(Collectors.joining("\n"));
        String format = StrUtil.format(langChainChatPrompt, Map.of("context", relevantDocument, "question", question));

        Prompt prompt = new Prompt(new UserMessage(format));

        log.info("开始询问GPT问题");
        ChatResponse call = ollamaChatModel.call(prompt);
        log.info("AI responded.");
        RagResVO ragResVO = new RagResVO();
        ragResVO.setAnswer(call.getResult().getOutput().getContent());
        ragResVO.setFileName(fileNameList);

        return ragResVO;
    }
}


